/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.channel;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.ChunkFactoryReference;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.protocol.ExportIdentifierChunk;
import org.objectweb.dream.protocol.IPExportIdentifier;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * This message transformer adds a {@link ExportIdentifierChunk} containing an
 * {@link IPExportIdentifier}. <br>
 * The name of the added chunk is specified using
 * {@link AddIPExportIdChunkAttributeController#setChunkName(String)}.<br>
 * The inet address must be specified using the
 * {@link AddIPExportIdChunkAttributeController#setHostname(String)} method.
 */
public class AddIPExportIdChunkImpl extends AbstractComponent
    implements
      Push,
      AddIPExportIdChunkAttributeController
{
  protected IPExportIdentifier                           exportIdentifier;
  protected ChunkFactoryReference<ExportIdentifierChunk> exportIdChunkFactory;

  // ---------------------------------------------------------------------------
  // Attributes
  // ---------------------------------------------------------------------------

  protected String                                       chunkName;
  protected String                                       hostname;
  protected int                                          port;

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  protected Push                                         outPushItf;
  protected MessageManager                               messageManagerItf;

  // ---------------------------------------------------------------------------
  // Implementation of Push interface
  // ---------------------------------------------------------------------------

  /**
   * @see Push#push(Message)
   */
  public void push(Message message) throws PushException
  {
    if (exportIdentifier == null)
    {
      exportIdentifier = new IPExportIdentifier(hostname, port);
    }
    if (exportIdChunkFactory == null)
    {
      exportIdChunkFactory = messageManagerItf
          .getChunkFactory(ExportIdentifierChunk.class);
    }
    ExportIdentifierChunk chunk = messageManagerItf
        .createChunk(exportIdChunkFactory);
    chunk.setExportIdentifier(exportIdentifier);
    messageManagerItf.addChunk(message, chunkName, chunk);
    outPushItf.push(message);
  }

  // ---------------------------------------------------------------------------
  // Implementation of AddIPExportIdChunkAttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see AddIPExportIdChunkAttributeController#getChunkName()
   */
  public String getChunkName()
  {
    return chunkName;
  }

  /**
   * @see AddIPExportIdChunkAttributeController#setChunkName(String)
   */
  public void setChunkName(String chunkName)
  {
    this.chunkName = chunkName;
  }

  /**
   * @see AddIPExportIdChunkAttributeController#getHostname()
   */
  public String getHostname()
  {
    return hostname;
  }

  /**
   * @see AddIPExportIdChunkAttributeController#setHostname(String)
   */
  public void setHostname(String hostname)
  {
    this.hostname = hostname;
    exportIdentifier = null;
  }

  /**
   * @see AddIPExportIdChunkAttributeController#getPort()
   */
  public int getPort()
  {
    return port;
  }

  /**
   * @see AddIPExportIdChunkAttributeController#setPort(int)
   */
  public void setPort(int port)
  {
    this.port = port;
    exportIdentifier = null;
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{OUT_PUSH_ITF_NAME, MessageManager.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(Push.OUT_PUSH_ITF_NAME))
    {
      outPushItf = (Push) serverItf;
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }
}
