/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.protocol.bus.logicalClock;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import org.objectweb.dream.message.AbstractChunk;
import org.objectweb.dream.util.Util;

/**
 * Chunk added by used by Logical clock protocol.
 */
public final class LogicalClockChunk extends AbstractChunk<LogicalClockChunk>
{

  private static final long  serialVersionUID = 3545798792923067701L;

  /** The default name for chunks of this type */
  public static final String DEFAULT_NAME     = "logical-clock";

  private int                from;
  private Object             logicalClock;

  /**
   * If the given <code>chunkName</code> is equals to the default name returns
   * {@link #DEFAULT_NAME }, returns the given string otherwise.
   * 
   * @param chunkName the chunk name to filter.
   * @return {@link #DEFAULT_NAME } or the given <code>chunkName</code>.
   */
  public static String filterChunkName(String chunkName)
  {
    if (DEFAULT_NAME.equals(chunkName))
    {
      return DEFAULT_NAME;
    }
    return chunkName;
  }

  // ---------------------------------------------------------------------------
  // Chunk methods
  // ---------------------------------------------------------------------------

  /**
   * Returns the logical clock of this chunk
   * 
   * @return the logical clock of this chunk
   */
  public Object getLogicalClock()
  {
    return logicalClock;
  }

  /**
   * Sets the logical clock of this chunk
   * 
   * @param stamp the stamp
   */
  public void setLogicalClock(Object stamp)
  {
    logicalClock = stamp;
  }

  /**
   * Returns the address of the process that emit this chunk.
   * 
   * @return the address of the process that emit this chunk.
   */
  public int getFrom()
  {
    return from;
  }

  /**
   * Sets the address of the process that emit this chunk.
   * 
   * @param from the address of the process that emit this chunk.
   */
  public void setFrom(int from)
  {
    this.from = from;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Chunk interface
  // ---------------------------------------------------------------------------

  /**
   * @see AbstractChunk#transfertStateTo(AbstractChunk)
   */
  public void transfertStateTo(LogicalClockChunk newInstance)
  {
    newInstance.logicalClock = logicalClock;
    newInstance.from = from;
  }

  /**
   * @see AbstractChunk#newChunk()
   */
  public LogicalClockChunk newChunk()
  {
    return new LogicalClockChunk();
  }

  /**
   * @see org.objectweb.dream.pool.Recyclable#recycle()
   */
  public void recycle()
  {
    from = 0;
    logicalClock = null;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Externalizable method
  // ---------------------------------------------------------------------------

  /**
   * @see java.io.Externalizable#readExternal(ObjectInput)
   */
  public void readExternal(ObjectInput in) throws IOException,
      ClassNotFoundException
  {
    from = in.readInt();
    logicalClock = Util.readObject(in);
  }

  /**
   * @see java.io.Externalizable#writeExternal(ObjectOutput)
   */
  public void writeExternal(ObjectOutput out) throws IOException
  {
    out.writeInt(from);
    Util.writeObject(out, logicalClock);
  }

}
