/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.channel;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import org.objectweb.dream.protocol.ExportIdentifier;
import org.objectweb.dream.protocol.Util;

/**
 * Export identifier created by multiplex protocol.
 */
public final class MultiplexExportIdentifier
    implements
      ExportIdentifier,
      Externalizable
{

  private static final long  serialVersionUID = 1L;
  private short              routeId;
  private ExportIdentifier[] next;

  /**
   * @param id the routeId
   * @param next identifier corresponding to lower level protocol layer.
   */
  public MultiplexExportIdentifier(short id, ExportIdentifier next)
  {
    routeId = id;
    this.next = new ExportIdentifier[]{next};
  }

  /**
   * @return Returns the routeId.
   */
  public short getRouteId()
  {
    return routeId;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the ExportIdentifier interface
  // ---------------------------------------------------------------------------

  /**
   * @see ExportIdentifier#getNextExportIds()
   */
  public ExportIdentifier[] getNextExportIds()
  {
    return next;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Externalizable interface
  // ---------------------------------------------------------------------------

  /**
   * @see Externalizable#readExternal(ObjectInput)
   */
  public void readExternal(ObjectInput in) throws IOException,
      ClassNotFoundException
  {
    routeId = in.readShort();
    next = Util.readExportIdentifierArray(in);
  }

  /**
   * @see Externalizable#writeExternal(ObjectOutput)
   */
  public void writeExternal(ObjectOutput out) throws IOException
  {
    out.writeShort(routeId);
    Util.writeExportIdentifierArray(out, next);
  }

  // ---------------------------------------------------------------------------
  // Overridden methods.
  // ---------------------------------------------------------------------------

  /**
   * @see Object#equals(Object)
   */
  public boolean equals(Object obj)
  {
    if (obj == this)
    {
      return true;
    }

    if (obj == null || !(obj instanceof MultiplexExportIdentifier))
    {
      return false;
    }
    MultiplexExportIdentifier mid = (MultiplexExportIdentifier) obj;
    ExportIdentifier[] nextExportIds = mid.getNextExportIds();
    if (routeId != mid.routeId || nextExportIds.length != 1)
    {
      return false;
    }

    return next[0].equals(nextExportIds[0]);
  }

  /**
   * @see Object#hashCode()
   */
  public int hashCode()
  {
    return routeId;
  }

  /**
   * @see Object#toString()
   */
  public String toString()
  {
    return "MultiplexExportIdentifier routeId=" + routeId;
  }

}
