/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Romain Lenglet
 */

package org.objectweb.dream.pump;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.control.activity.task.AbstractTask;
import org.objectweb.dream.control.activity.task.Task;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.synchro.Mutex;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Abstract implementation of a synchronized Pull/Push pump. This component has
 * an optional {@link org.objectweb.dream.synchro.Mutex} client interface that
 * it uses to synchronize the execution of its run method with other components.
 */
public abstract class AbstractPumpSynchronizedImpl extends AbstractComponent
    implements
      PumpAttributeController
{

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  protected Push           outPushItf;
  protected Pull           inPullItf;
  protected Mutex          mutexItf;
  protected MessageManager messageManagerItf;

  // ---------------------------------------------------------------------------
  // Attributes
  // ---------------------------------------------------------------------------

  protected boolean        pushNullPolicy            = false;

  protected boolean        stopOnPushExceptionPolicy = true;

  protected boolean        stopOnPullExceptionPolicy = true;

  // ---------------------------------------------------------------------------
  // Fields
  // ---------------------------------------------------------------------------

  protected AbstractTask   pumpTask                  = new PumpTask();

  // ---------------------------------------------------------------------------
  // Implementation of PumpTask class
  // ---------------------------------------------------------------------------

  class PumpTask extends AbstractTask
  {

    /**
     * Default constructor
     */
    public PumpTask()
    {
      super("PumpTask");
    }

    /**
     * @see org.objectweb.dream.control.activity.task.Task#execute(Object)
     */
    public Object execute(Object hints) throws InterruptedException
    {
      if (mutexItf != null)
      {
        mutexItf.lock();
      }
      Message msg = null;
      try
      {
        msg = inPullItf.pull();
        if (msg != null || pushNullPolicy)
        {
          outPushItf.push(msg);
        }
      }
      catch (PullException e)
      {
        return handlePullException(e);
      }
      catch (PushException e)
      {
        return handlePushException(msg, e);
      }
      finally
      {
        if (mutexItf != null)
        {
          mutexItf.unlock();
        }
      }
      return EXECUTE_AGAIN;
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the PumpAttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see PumpAttributeController#getPushNullPolicy()
   */
  public boolean getPushNullPolicy()
  {
    return pushNullPolicy;
  }

  /**
   * @see PumpAttributeController#setPushNullPolicy(boolean)
   */
  public void setPushNullPolicy(boolean pushNull)
  {
    pushNullPolicy = pushNull;
  }

  /**
   * @see PumpAttributeController#getStopOnPushExceptionPolicy()
   */
  public boolean getStopOnPushExceptionPolicy()
  {
    return stopOnPushExceptionPolicy;
  }

  /**
   * @see PumpAttributeController#setStopOnPushExceptionPolicy(boolean)
   */
  public void setStopOnPushExceptionPolicy(boolean stopOnPushException)
  {
    this.stopOnPushExceptionPolicy = stopOnPushException;
  }

  /**
   * @see PumpAttributeController#getStopOnPullExceptionPolicy()
   */
  public boolean getStopOnPullExceptionPolicy()
  {
    return stopOnPullExceptionPolicy;
  }

  /**
   * @see PumpAttributeController#setStopOnPullExceptionPolicy(boolean)
   */
  public void setStopOnPullExceptionPolicy(boolean stopOnPullException)
  {
    this.stopOnPullExceptionPolicy = stopOnPullException;
  }

  // ---------------------------------------------------------------------------
  // Utility methods.
  // ---------------------------------------------------------------------------

  /**
   * Handles a {@link PushException} occurring when the activity tries to push a
   * message. This implementation simply deletes the message using message
   * manger.
   * 
   * @param message the pushed message
   * @param exception the exception thrown during the push call.
   */
  protected Object handlePushException(Message message, PushException exception)
  {
    messageManagerItf.deleteMessage(message);
    if (stopOnPushExceptionPolicy)
    {
      if (logger.isLoggable(BasicLevel.WARN))
      {
        logger.log(BasicLevel.WARN,
            "Push exception occurred, stop executing the task", exception);
      }
      return Task.STOP_EXECUTING;
    }
    if (logger.isLoggable(BasicLevel.WARN))
    {
      logger.log(BasicLevel.WARN,
          "Push exception occurred, continue executing the task", exception);
    }
    return Task.EXECUTE_AGAIN;
  }

  /**
   * Handles a {@link PullException} occurring when the activity tries to pull a
   * message.
   * 
   * @param exception the exception thrown during the pull call.
   */
  protected Object handlePullException(PullException exception)
  {
    if (stopOnPullExceptionPolicy)
    {
      if (logger.isLoggable(BasicLevel.WARN))
      {
        logger.log(BasicLevel.WARN,
            "Pull exception occurred, stop executing the task", exception);
      }
      return AbstractTask.STOP_EXECUTING;
    }
    if (logger.isLoggable(BasicLevel.WARN))
    {
      logger.log(BasicLevel.WARN,
          "Pull exception occurred, continue executing the task", exception);
    }
    return AbstractTask.EXECUTE_AGAIN;
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Push.OUT_PUSH_ITF_NAME, Pull.IN_PULL_ITF_NAME,
        Mutex.ITF_NAME, MessageManager.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(Push.OUT_PUSH_ITF_NAME))
    {
      outPushItf = (Push) serverItf;
    }
    else if (clientItfName.equals(Pull.IN_PULL_ITF_NAME))
    {
      inPullItf = (Pull) serverItf;
    }
    else if (clientItfName.equals(Mutex.ITF_NAME))
    {
      mutexItf = (Mutex) serverItf;
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

}
