/**
 * DREAM
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.pushwithreturn;

import java.lang.ref.Reference;

import org.objectweb.dream.message.Message;
import org.objectweb.dream.pool.Recyclable;

/**
 * This class represents a waiting key. A waiting key stores data associated to
 * a key: a message, a boolean indicating whether a returned message
 * corresponding to the key has been received.
 */
public class WaitingKey implements Recyclable
{
  /** The return message. */
  private Message                          returnMessage;
  private Message                          message;
  private Key                              key;
  private boolean                          returnMessageReceived = false;
  private Reference< ? extends Recyclable> recyclableReference;

  /**
   * Returns the return message.
   * 
   * @return Returns the message.
   */
  public Message getReturnMessage()
  {
    return returnMessage;
  }

  /**
   * Sets the return message.
   * 
   * @param returnMessage the return message to set.
   */
  public void setReturnMessage(Message returnMessage)
  {
    this.returnMessage = returnMessage;
  }

  /**
   * Sets the returnMessageReceived boolean to the specifed value.
   * 
   * @param returnMessageReceived the value to set.
   */
  public void setReturnMessageReceived(boolean returnMessageReceived)
  {
    this.returnMessageReceived = returnMessageReceived;
  }

  /**
   * Returns <code>true</code> if the return message has been received.
   * 
   * @return <code>true</code> if the return message has been received.
   */
  public boolean getReturnMessageReceived()
  {
    return returnMessageReceived;
  }

  /**
   * Sets the message associated to this key.
   * 
   * @param message the message to be set.
   */
  public void setMessage(Message message)
  {
    this.message = message;
  }

  /**
   * Returns the message associated to this key.
   * 
   * @return the message associated to this key.
   */
  public Message getMessage()
  {
    return message;
  }

  /**
   * Sets the key associated to this waiting key.
   * 
   * @param key the key to be set.
   */
  public void setKey(Key key)
  {
    this.key = key;
  }

  /**
   * Returns the key associated to this WaitingKey.
   * 
   * @return the key associated to this WaitingKey.
   */
  public Key getKey()
  {
    return key;
  }

  // -------------------------------------------------------------------------
  // Implementation of the Recyclable interface.
  // -------------------------------------------------------------------------

  /**
   * @see Recyclable#recycle()
   */
  public void recycle()
  {
    this.returnMessage = null;
    this.returnMessageReceived = false;
    this.key = null;
    this.message = null;
  }

  /**
   * @see Recyclable#getReference()
   */
  public Reference< ? extends Recyclable> getReference()
  {
    return recyclableReference;
  }

  /**
   * @see Recyclable#setReference(Reference)
   */
  public void setReference(Reference< ? extends Recyclable> recyclableReference)
  {
    this.recyclableReference = recyclableReference;
  }

}
