/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Romain Lenglet
 * Contributor(s):
 */

package org.objectweb.dream.queue.keyed;

import org.objectweb.dream.message.Message;

/**
 * This interface defines methods that must be implemented by a buffer to allow
 * components to get or delete messages from it, according to the key associated
 * with each message. Such an interface is used by <i>consumers </i> of
 * messages. <p/><i>Note 1: </i> all the messages that are stored in a buffer
 * are not always available for removal (e.g. when the buffer implements an
 * ordering policy). <p/><i>Note 2: </i> <b>buffer implementations MUST BE
 * thread-safe </b>.
 */
public interface KeyedBufferRemove
{

  /**
   * The commonly used name to refer to an interface with this signature.
   */
  String ITF_NAME = "keyed-buffer-remove";

  /**
   * Gets the message from the buffer that is associated with the specified key
   * object. This method blocks while no message associated with the specified
   * key is available.
   * 
   * @param key the key that is associated with the message to get.
   * @return the message that is associated with the specified key; never
   *         <code>null</code>.
   * @throws InterruptedException if it is interrupted while waiting for a
   *           message to be added.
   * @see org.objectweb.dream.queue.BufferRemove#get()
   */
  Message get(Object key) throws InterruptedException;

  /**
   * Gets the message from the buffer that is associated with the specified key
   * object. This method does not block.
   * 
   * @param key the key that is associated with the message to get.
   * @return the message that is associated with the specified key if there is
   *         one, <code>null</code> otherwise.
   * @see org.objectweb.dream.queue.BufferRemove#tryGet()
   */
  Message tryGet(Object key);

  /**
   * Removes the message from the buffer that is associated with the specified
   * key object. This method blocks while no message associated with the
   * specified key is available.
   * 
   * @param key the key that is associated with the message to be removed.
   * @return the removed message that was associated with the specified key;
   *         never <code>null</code>.
   * @throws InterruptedException if it is interrupted while waiting for a
   *           message to be added.
   * @see org.objectweb.dream.queue.BufferRemove#remove()
   */
  Message remove(Object key) throws InterruptedException;

  /**
   * Removes the message from the buffer that is associated with the specified
   * key object. This method does not block.
   * 
   * @param key the key that is associated with the message to be removed.
   * @return the removed message that was associated with the specified key if
   *         there was one, <code>null</code> otherwise.
   * @see org.objectweb.dream.queue.BufferRemove#tryRemove()
   */
  Message tryRemove(Object key);

  /**
   * Removes all the available messages from the buffer. Messages are returned
   * in an aggregated message. If no message has been removed, this method
   * returns <code>null</code>. This method does not block.
   * 
   * @return an aggregated message containing all the messages available from
   *         the buffer, or <code>null</code> if no message was available.
   * @see org.objectweb.dream.queue.BufferRemove#removeAll()
   */
  Message removeAll();

}