/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Romain Lenglet
 */

package org.objectweb.dream.queue.keyed;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.queue.BufferAdd;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * A, incoming handler that implements the {@link KeyedPush}interface. It adds
 * {@link #push(Message, Object) pushed}messages into a
 * {@link BufferAdd buffer}bound to it. This component uses
 * {@link BufferAdd#add(Message) blocking calls for adding}messages into the
 * buffer.
 */
public class KeyedIncomingHandlerBlockingImpl extends AbstractComponent
    implements
      KeyedPush
{

  // ---------------------------------------------------------------------------
  // Constants.
  // ---------------------------------------------------------------------------

  /**
   * The names of client interfaces.
   */
  protected static final String[] CLIENT_INTERFACE_NAMES = {KeyedBufferAdd.ITF_NAME};

  // ---------------------------------------------------------------------------
  // Client interfaces.
  // ---------------------------------------------------------------------------

  /**
   * The buffer into which pushed messages are added.
   */
  protected KeyedBufferAdd        keyedBufferAddItf;

  // ---------------------------------------------------------------------------
  // Implementation of the KeyedPush interface.
  // ---------------------------------------------------------------------------

  /**
   * @see KeyedPush#push(Message, Object)
   */
  public void push(Message message, Object key) throws PushException
  {
    try
    {
      keyedBufferAddItf.add(message, key);
    }
    catch (InterruptedException e)
    {
      throw new PushException(
          "interrupted while adding message into the buffer", e);
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return CLIENT_INTERFACE_NAMES;
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(BufferAdd.ITF_NAME))
    {
      keyedBufferAddItf = (KeyedBufferAdd) serverItf;
    }
  }

}