/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.channel;

import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.protocol.BindException;
import org.objectweb.dream.protocol.ExportIdentifier;
import org.objectweb.dream.protocol.IPExportIdentifier;
import org.objectweb.dream.protocol.IncomingPush;
import org.objectweb.dream.protocol.InvalidExportIdentifierException;
import org.objectweb.dream.protocol.OutgoingPush;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;

public class TestMultiplexExportProtocol extends TestCase
{

  protected Component       protocolComponent;
  protected ChannelProtocol serverProtocolItf;
  protected ChannelProtocol clientProtocolItf;
  protected MessageManager  messageManagerItf;

  protected Channel         channel1;
  protected Channel         channel2;

  protected void setUp() throws Exception
  {
    super.setUp();
    protocolComponent = (Component) FactoryFactory.getFactory().newComponent(
        getClass().getName(), new HashMap());
    Fractal.getLifeCycleController(protocolComponent).startFc();
    serverProtocolItf = (ChannelProtocol) protocolComponent
        .getFcInterface("server-protocol");
    clientProtocolItf = (ChannelProtocol) protocolComponent
        .getFcInterface("client-protocol");
    messageManagerItf = (MessageManager) protocolComponent
        .getFcInterface(MessageManager.ITF_NAME);
    channel1 = new Channel();
    channel2 = new Channel();
  }

  public void testExportBindCloseUnexport() throws Exception
  {
    // exports
    MultiplexExportIdentifier exportId1 = (MultiplexExportIdentifier) serverProtocolItf
        .export(channel1, null);
    MultiplexExportIdentifier exportId2 = (MultiplexExportIdentifier) serverProtocolItf
        .export(channel2, null);
    assertEquals(1, exportId1.getNextExportIds().length);
    assertEquals(1, exportId2.getNextExportIds().length);
    assertEquals(exportId1.getNextExportIds()[0],
        exportId2.getNextExportIds()[0]);

    // binds
    OutgoingPush outgoingPush1 = clientProtocolItf.bind(exportId1, null, null);
    channel1.waitInstantiate();

    OutgoingPush outgoingPush2 = clientProtocolItf.bind(exportId2, null, null);
    channel1.waitInstantiate();

    // push messages.
    outgoingPush1.outgoingPush(messageManagerItf.createMessage());
    channel1.serverSession.waitMessage();

    outgoingPush2.outgoingPush(messageManagerItf.createMessage());
    channel2.serverSession.waitMessage();

    // close sessions.
    outgoingPush1.outgoingClose(null);
    channel1.serverSession.waitClosed();
    channel1.serverSession = null;

    outgoingPush2.outgoingClose(null);
    channel2.serverSession.waitClosed();
    channel2.serverSession = null;

    // unexport 1
    serverProtocolItf.unexport(exportId1);
    Thread.sleep(1000);
    // try to rebind
    try
    {
      clientProtocolItf.bind(exportId1, null, null);
      fail("Able to bind after unexport");
    }
    catch (Exception e)
    {
      // OK
    }

    // make sure that 2 is always exported
    outgoingPush2 = clientProtocolItf.bind(exportId2, null, null);
    channel2.waitInstantiate();

    // unexport 2
    serverProtocolItf.unexport(exportId2);
    Thread.sleep(1000);
    // try to rebind
    try
    {
      clientProtocolItf.bind(exportId2, null, null);
      fail("Able to bind after unexport");
    }
    catch (Exception e)
    {
      // OK
    }
  }

  public void testExportBindCloseServerSideUnexport() throws Exception
  {
    ExportIdentifier exportIdentifier = serverProtocolItf
        .export(channel1, null);

    // bind to export
    Session clientSession = new Session();
    OutgoingPush outgoingPush = clientProtocolItf.bind(exportIdentifier,
        clientSession, null);
    clientSession.outgoingPush = outgoingPush;
    channel1.waitInstantiate();

    // close at server side
    channel1.serverSession.outgoingPush.outgoingClose(channel1.serverSession);
    clientSession.waitClosed();

    // unexport.
    serverProtocolItf.unexport(exportIdentifier);
  }

  public void testExportBindExceptionAtInstantiate() throws Exception
  {
    ExportIdentifier exportIdentifier = serverProtocolItf.export(
        new ChannelFactory()
        {

          public IncomingPush instantiate(OutgoingPush toClientPush)
              throws BindException
          {
            throw new BindException("test exception");
          }
        }, null);

    // bind to export
    try
    {
      clientProtocolItf.bind(exportIdentifier, null, null);
      fail("bind should throw an exception here");
    }
    catch (BindException e)
    {
      assertEquals("test exception", e.getCause().getMessage());
    }

    // unexport.
    serverProtocolItf.unexport(exportIdentifier);
  }

  public void testExportBindNullAtInstantiate() throws Exception
  {
    ExportIdentifier exportIdentifier = serverProtocolItf.export(
        new ChannelFactory()
        {

          public IncomingPush instantiate(OutgoingPush toClientPush)
              throws BindException
          {
            return null;
          }
        }, null);

    // bind to export
    OutgoingPush outgoingPush = clientProtocolItf.bind(exportIdentifier, null,
        null);
    outgoingPush.outgoingClose(null);

    // unexport.
    serverProtocolItf.unexport(exportIdentifier);
  }

  public void testCreateExportIdentifier1() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(MultiplexExportProtocol.ROUTE_ID, new Integer(1));
    ExportIdentifier lowerId = new IPExportIdentifier();
    ExportIdentifier exportIdentifier = serverProtocolItf
        .createExportIdentifier(hints, new ExportIdentifier[]{lowerId});

    assertTrue(exportIdentifier instanceof MultiplexExportIdentifier);
    MultiplexExportIdentifier id = (MultiplexExportIdentifier) exportIdentifier;

    assertEquals(1, id.getRouteId());
    assertNotNull(id.getNextExportIds());
    assertEquals(1, id.getNextExportIds().length);
    assertEquals(lowerId, id.getNextExportIds()[0]);
  }

  public void testCreateExportIdentifier2() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    ExportIdentifier lowerId = new IPExportIdentifier();
    try
    {
      serverProtocolItf.createExportIdentifier(hints,
          new ExportIdentifier[]{lowerId});
      fail("createExportIdentifier should throw an exception here");
    }
    catch (InvalidExportIdentifierException e)
    {
      // OK
    }
  }

  public void testCreateExportIdentifier3() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(MultiplexExportProtocol.ROUTE_ID, new Integer(1));
    try
    {
      serverProtocolItf.createExportIdentifier(hints, null);
      fail("createExportIdentifier should throw an exception here");
    }
    catch (InvalidExportIdentifierException e)
    {
      // OK
    }
  }

  public void testCreateExportIdentifier4() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    try
    {
      serverProtocolItf.createExportIdentifier(hints, null);
      fail("createExportIdentifier should throw an exception here");
    }
    catch (InvalidExportIdentifierException e)
    {
      // OK
    }
  }

  public void testCreateExportIdentifier5() throws Exception
  {
    try
    {
      serverProtocolItf.createExportIdentifier(null, null);
      fail("createExportIdentifier should throw an exception here");
    }
    catch (InvalidExportIdentifierException e)
    {
      // OK
    }
  }

}
