/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.channel;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.objectweb.dream.IOPushException;
import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.dream.message.AbstractChunk;
import org.objectweb.dream.message.ChunkFactoryReference;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.protocol.BindException;
import org.objectweb.dream.protocol.ExceptionChunk;
import org.objectweb.dream.protocol.ExportIdentifier;
import org.objectweb.dream.protocol.IPExportIdentifier;
import org.objectweb.dream.protocol.IncomingPush;
import org.objectweb.dream.protocol.InvalidExportIdentifierException;
import org.objectweb.dream.protocol.OutgoingPush;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;

public class TestTCPIPProtocol extends TestCase
{

  private final static int                        BIND_CLOSE_LOOP_ITERATION      = 10000;
  private final static int                        EXPORT_UNEXPORT_LOOP_ITERATION = 10000;

  protected Component                             protocolComponent;
  protected TCPIPProtocol                         serverProtocolItf;
  protected TCPIPProtocol                         clientProtocolItf;
  protected MessageManager                        messageManagerItf;
  protected ChunkFactoryReference<ExceptionChunk> exceptionChunkFactoryReference;

  protected Channel                               testChannel;

  protected void setUp() throws Exception
  {
    protocolComponent = (Component) FactoryFactory.getFactory().newComponent(
        getClass().getName(), new HashMap());
    Fractal.getLifeCycleController(protocolComponent).startFc();
    serverProtocolItf = (TCPIPProtocol) protocolComponent
        .getFcInterface("server-protocol");
    clientProtocolItf = (TCPIPProtocol) protocolComponent
        .getFcInterface("client-protocol");
    messageManagerItf = (MessageManager) protocolComponent
        .getFcInterface(MessageManager.ITF_NAME);
    exceptionChunkFactoryReference = messageManagerItf
        .getChunkFactory(ExceptionChunk.class);
    testChannel = new Channel();
  }

  public void testExportBindCloseUnexport() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(TCPIPProtocol.PORT, new Integer(10000));
    ExportIdentifier exportIdentifier = serverProtocolItf.export(testChannel,
        hints);

    // bind to new export
    OutgoingPush outgoingPush = clientProtocolItf.bind(exportIdentifier, null,
        null);
    testChannel.waitInstantiate();

    // push a message.
    outgoingPush.outgoingPush(messageManagerItf.createMessage());
    testChannel.serverSession.waitMessage();

    // close session.
    outgoingPush.outgoingClose(null);
    testChannel.serverSession.waitClosed();
    testChannel.serverSession = null;

    // unexport.
    serverProtocolItf.unexport(exportIdentifier);
    Thread.sleep(1000);
    try
    {
      // try to rebind
      clientProtocolItf.bind(exportIdentifier, null, null);
      fail("Able to bind after unexport");
    }
    catch (Exception e)
    {
      // OK
    }
  }

  public void testExportBindCloseServerSideUnexport() throws Exception
  {
    ExportIdentifier exportIdentifier = serverProtocolItf.export(testChannel,
        null);

    // bind to export
    Session clientSession = new Session();
    OutgoingPush outgoingPush = clientProtocolItf.bind(exportIdentifier,
        clientSession, null);
    clientSession.outgoingPush = outgoingPush;
    testChannel.waitInstantiate();

    // close at server side
    testChannel.serverSession.outgoingPush
        .outgoingClose(testChannel.serverSession);
    clientSession.waitClosed();
    testChannel.serverSession = null;

    // unexport.
    serverProtocolItf.unexport(exportIdentifier);
    Thread.sleep(1000);
    try
    {
      // try to rebind
      clientProtocolItf.bind(exportIdentifier, null, null);
      fail("Able to bind after unexport");
    }
    catch (Exception e)
    {
      // OK
    }
  }

  public void testMessageExchange() throws Exception
  {
    ExportIdentifier exportIdentifier = serverProtocolItf.export(testChannel,
        null);

    // bind to new export
    Session clientSession = new Session();
    OutgoingPush outgoingPush = clientProtocolItf.bind(exportIdentifier,
        clientSession, null);
    testChannel.waitInstantiate();

    // push message from client to server.
    Message message = messageManagerItf.createMessage();
    // use exception chunk as content of message.
    Exception testException = new IOException("just for test");
    ExceptionChunk exceptionChunk = messageManagerItf
        .createChunk(exceptionChunkFactoryReference);
    exceptionChunk.setException(testException);
    messageManagerItf.addChunk(message, "toto", exceptionChunk);
    outgoingPush.outgoingPush(message);
    Message receivedMessage = testChannel.serverSession.waitMessage();
    AbstractChunk chunk = messageManagerItf.getChunk(receivedMessage, "toto");
    assertNotNull(chunk);
    assertTrue(chunk instanceof ExceptionChunk);
    assertNotNull(((ExceptionChunk) chunk).getException());
    assertEquals(testException.getMessage(), ((ExceptionChunk) chunk)
        .getException().getMessage());

    // reply from server to client.
    message = messageManagerItf.createMessage();
    testException = new IOException("just a reply");
    exceptionChunk = messageManagerItf
        .createChunk(exceptionChunkFactoryReference);
    exceptionChunk.setException(testException);
    messageManagerItf.addChunk(message, "titi", exceptionChunk);
    testChannel.serverSession.outgoingPush.outgoingPush(message);
    receivedMessage = clientSession.waitMessage();
    chunk = messageManagerItf.getChunk(receivedMessage, "titi");
    assertNotNull(chunk);
    assertTrue(chunk instanceof ExceptionChunk);
    assertNotNull(((ExceptionChunk) chunk).getException());
    assertEquals(testException.getMessage(), ((ExceptionChunk) chunk)
        .getException().getMessage());

    // close client side
    outgoingPush.outgoingClose(clientSession);
    testChannel.serverSession.waitClosed();

    // try to send message on client and server side
    try
    {
      outgoingPush.outgoingPush(messageManagerItf.createMessage());
      fail("outgoingPush should throw an exception here");
    }
    catch (IOPushException e)
    {
      // ok
    }

    try
    {
      testChannel.serverSession.outgoingPush.outgoingPush(messageManagerItf
          .createMessage());
      fail("outgoingPush should throw an exception here");
    }
    catch (IOPushException e)
    {
      // ok
    }

    // unexport
    serverProtocolItf.unexport(exportIdentifier);
  }

  public void testExportBindExceptionAtInstantiate() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(TCPIPProtocol.PORT, new Integer(10000));
    ExportIdentifier exportIdentifier = serverProtocolItf.export(
        new ChannelFactory()
        {

          public IncomingPush instantiate(OutgoingPush toClientPush)
              throws BindException
          {
            throw new BindException("test exception");
          }
        }, hints);

    // bind to export
    try
    {
      clientProtocolItf.bind(exportIdentifier, null, null);
      fail("bind should throw an exception here");
    }
    catch (BindException e)
    {
      assertEquals("test exception", e.getCause().getMessage());
    }

    // unexport.
    serverProtocolItf.unexport(exportIdentifier);
    Thread.sleep(1000);
    try
    {
      // try to rebind
      clientProtocolItf.bind(exportIdentifier, null, null);
      fail("Able to bind after unexport");
    }
    catch (Exception e)
    {
      // OK
    }
  }

  public void testExportBindNullAtInstantiate() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(TCPIPProtocol.PORT, new Integer(10000));
    ExportIdentifier exportIdentifier = serverProtocolItf.export(
        new ChannelFactory()
        {

          public IncomingPush instantiate(OutgoingPush toClientPush)
              throws BindException
          {
            return null;
          }
        }, hints);

    // bind to export
    OutgoingPush outgoingPush = clientProtocolItf.bind(exportIdentifier, null,
        null);
    outgoingPush.outgoingClose(null);

    // unexport.
    serverProtocolItf.unexport(exportIdentifier);
    Thread.sleep(1000);
    try
    {
      // try to rebind
      clientProtocolItf.bind(exportIdentifier, null, null);
      fail("Able to bind after unexport");
    }
    catch (Exception e)
    {
      // OK
    }

  }

  public void testExportBindCloseLoop() throws Exception
  {
    ExportIdentifier exportIdentifier = serverProtocolItf.export(testChannel,
        null);

    for (int i = 0; i < BIND_CLOSE_LOOP_ITERATION; i++)
    {
      // bind to new export
      OutgoingPush outgoingPush = clientProtocolItf.bind(exportIdentifier,
          null, null);
      synchronized (testChannel)
      {
        if (testChannel.serverSession == null)
        {
          testChannel.wait(1000);
        }
        assertNotNull(testChannel.serverSession);
      }

      // close session.
      outgoingPush.outgoingClose(null);
      testChannel.serverSession.waitClosed();
      testChannel.serverSession = null;
    }
    // unexport.
    serverProtocolItf.unexport(exportIdentifier);
    Thread.sleep(1000);
    try
    {
      // try to rebind
      clientProtocolItf.bind(exportIdentifier, null, null);
      fail("Able to bind after unexport");
    }
    catch (Exception e)
    {
      // OK
    }
  }

  public void testExportUnexportLoop() throws Exception
  {
    for (int i = 0; i < EXPORT_UNEXPORT_LOOP_ITERATION; i++)
    {
      ExportIdentifier exportIdentifier = serverProtocolItf.export(testChannel,
          null);
      // unexport.
      serverProtocolItf.unexport(exportIdentifier);
    }
  }

  public void testCreateExportIdentifier1() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(TCPIPProtocol.ADDRESS, "localhost");
    hints.put(TCPIPProtocol.PORT, new Integer(1000));
    IPExportIdentifier exportIdentifier = (IPExportIdentifier) serverProtocolItf
        .createExportIdentifier(hints, null);
    assertEquals("localhost", exportIdentifier.getHostName());
    assertEquals(1000, exportIdentifier.getPort());
  }

  public void testCreateExportIdentifier2() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(TCPIPProtocol.ADDRESS, "localhost");
    try
    {
      serverProtocolItf.createExportIdentifier(hints, null);
      fail("createExportIdentifier should throw an exception here");
    }
    catch (InvalidExportIdentifierException e)
    {
      // OK
    }
  }

  public void testCreateExportIdentifier3() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(TCPIPProtocol.PORT, new Integer(1000));
    try
    {
      serverProtocolItf.createExportIdentifier(hints, null);
      fail("createExportIdentifier should throw an exception here");
    }
    catch (InvalidExportIdentifierException e)
    {
      // OK
    }
  }

  public void testCreateExportIdentifier4() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    try
    {
      serverProtocolItf.createExportIdentifier(hints, null);
      fail("createExportIdentifier should throw an exception here");
    }
    catch (InvalidExportIdentifierException e)
    {
      // OK
    }
  }

  public void testCreateExportIdentifier5() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(TCPIPProtocol.ADDRESS, "localhost");
    hints.put(TCPIPProtocol.PORT, new Integer(1000));
    ExportIdentifier lowerId = new IPExportIdentifier();
    try
    {
      serverProtocolItf.createExportIdentifier(hints,
          new ExportIdentifier[]{lowerId});
      fail("createExportIdentifier should throw an exception here");
    }
    catch (InvalidExportIdentifierException e)
    {
      // OK
    }
  }

  public void testCreateExportIdentifier6() throws Exception
  {
    try
    {
      serverProtocolItf.createExportIdentifier(null, null);
      fail("createExportIdentifier should throw an exception here");
    }
    catch (InvalidExportIdentifierException e)
    {
      // OK
    }
  }

  public void testCreateExportIdentifier7() throws Exception
  {
    serverProtocolItf.createExportIdentifier("localhost", 1000);
  }
}
