/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.messagePassing;

import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.objectweb.dream.IOPushException;
import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.protocol.ExportIdentifier;
import org.objectweb.dream.protocol.IPExportIdentifier;
import org.objectweb.dream.protocol.InvalidExportIdentifierException;
import org.objectweb.dream.protocol.channel.TCPIPProtocol;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;

public class TestUDPProtocol extends TestCase
{
  private final static int EXPORT_CLOSE_LOOP_ITERATION = 10000;

  protected Component      protocolComponent;
  protected UDPProtocol    protocol1Itf;
  protected UDPProtocol    protocol2Itf;
  protected MessageManager messageManagerItf;

  protected void setUp() throws Exception
  {
    protocolComponent = (Component) FactoryFactory.getFactory().newComponent(
        getClass().getName(), new HashMap());
    Fractal.getLifeCycleController(protocolComponent).startFc();
    protocol1Itf = (UDPProtocol) protocolComponent.getFcInterface("protocol1");
    protocol2Itf = (UDPProtocol) protocolComponent.getFcInterface("protocol2");
    messageManagerItf = (MessageManager) protocolComponent
        .getFcInterface(MessageManager.ITF_NAME);
  }

  public void testExportPushClose() throws Exception
  {
    Session s1 = new Session();
    s1.outgoingPush = protocol1Itf.export(s1, null);

    Session s2 = new Session();
    s2.outgoingPush = protocol2Itf.export(s2, null);

    // push message from 1 to 2
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s2.outgoingPush.getLocalExportIdentifier());
    s2.waitMessage();

    // push message from 2 to 1
    s2.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s1.outgoingPush.getLocalExportIdentifier());
    s1.waitMessage();

    // close s2
    s2.outgoingPush.outgoingClose(s2);
    Thread.sleep(100);

    // try to push from 1 to 2
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s2.outgoingPush.getLocalExportIdentifier());
    synchronized (s2)
    {
      if (s2.receivedMessage == null)
      {
        s2.wait(500);
      }
      assertNull(s2.receivedMessage);
    }

    // close s1
    s1.outgoingPush.outgoingClose(s1);
    // try to push on closed 1
    try
    {
      s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
          s2.outgoingPush.getLocalExportIdentifier());
      fail("outgoingPush should throw an exception here");
    }
    catch (IOPushException e)
    {
      // ok
    }
  }

  public void testExportFromChunk1() throws Exception
  {
    Session s1 = new Session();
    s1.outgoingPush = protocol1Itf.export(s1, 0, 0, "from1");

    Session s2 = new Session();
    s2.outgoingPush = protocol2Itf.export(s2, 0, 0, "from2");

    // push message from 1 to 2
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s2.outgoingPush.getLocalExportIdentifier());
    Message receivedMessage = s2.waitMessage();
    Util.checkMessage(receivedMessage, "from2", s1.outgoingPush
        .getLocalExportIdentifier(), messageManagerItf);

    // push message from 2 to 1
    s2.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s1.outgoingPush.getLocalExportIdentifier());
    receivedMessage = s1.waitMessage();
    Util.checkMessage(receivedMessage, "from1", s2.outgoingPush
        .getLocalExportIdentifier(), messageManagerItf);

    // push message from 1 to 1
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s1.outgoingPush.getLocalExportIdentifier());
    receivedMessage = s1.waitMessage();
    Util.checkMessage(receivedMessage, "from1", s1.outgoingPush
        .getLocalExportIdentifier(), messageManagerItf);

    // push message from 2 to 2
    s2.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s2.outgoingPush.getLocalExportIdentifier());
    receivedMessage = s2.waitMessage();
    Util.checkMessage(receivedMessage, "from2", s2.outgoingPush
        .getLocalExportIdentifier(), messageManagerItf);

    s1.outgoingPush.outgoingClose(s1);
    s2.outgoingPush.outgoingClose(s2);
    Thread.sleep(100);
  }

  public void testExportFromChunk2() throws Exception
  {
    Map<String, Object> hints1 = new HashMap<String, Object>();
    hints1.put(UDPProtocol.FROM_CHUNK_NAME, "from1");
    hints1.put(UDPProtocol.PORT, new Integer(4000));
    Session s1 = new Session();
    s1.outgoingPush = protocol1Itf.export(s1, hints1);

    ExportIdentifier id1 = s1.outgoingPush.getLocalExportIdentifier();
    assertTrue(id1 instanceof IPExportIdentifier);
    assertEquals(4000, ((IPExportIdentifier) id1).getPort());

    Map<String, Object> hints2 = new HashMap<String, Object>();
    hints2.put(UDPProtocol.FROM_CHUNK_NAME, "from2");
    hints2.put(UDPProtocol.PORT, new Integer(4001));
    Session s2 = new Session();
    s2.outgoingPush = protocol2Itf.export(s2, hints2);

    ExportIdentifier id2 = s2.outgoingPush.getLocalExportIdentifier();
    assertTrue(id2 instanceof IPExportIdentifier);
    assertEquals(4001, ((IPExportIdentifier) id2).getPort());

    // push message from 1 to 2
    s1.outgoingPush.outgoingPush(messageManagerItf.createMessage(),
        s2.outgoingPush.getLocalExportIdentifier());
    Message receivedMessage = s2.waitMessage();
    Util.checkMessage(receivedMessage, "from2", id1, messageManagerItf);

    s1.outgoingPush.outgoingClose(s1);
    s2.outgoingPush.outgoingClose(s2);
  }

  public void testExportCloseLoop() throws Exception
  {
    Session session = new Session();
    for (int i = 0; i < EXPORT_CLOSE_LOOP_ITERATION; i++)
    {
      MessagePassingOutgoingPush outgoingPush = protocol1Itf.export(session,
          null);
      // unexport.
      outgoingPush.outgoingClose(session);
    }
  }

  public void testCreateExportIdentifier1() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(TCPIPProtocol.ADDRESS, "localhost");
    hints.put(TCPIPProtocol.PORT, new Integer(1000));
    IPExportIdentifier exportIdentifier = (IPExportIdentifier) protocol1Itf
        .createExportIdentifier(hints, null);
    assertEquals("localhost", exportIdentifier.getHostName());
    assertEquals(1000, exportIdentifier.getPort());
  }

  public void testCreateExportIdentifier2() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(TCPIPProtocol.ADDRESS, "localhost");
    try
    {
      protocol1Itf.createExportIdentifier(hints, null);
      fail("createExportIdentifier should throw an exception here");
    }
    catch (InvalidExportIdentifierException e)
    {
      // OK
    }
  }

  public void testCreateExportIdentifier3() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(TCPIPProtocol.PORT, new Integer(1000));
    try
    {
      protocol1Itf.createExportIdentifier(hints, null);
      fail("createExportIdentifier should throw an exception here");
    }
    catch (InvalidExportIdentifierException e)
    {
      // OK
    }
  }

  public void testCreateExportIdentifier4() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    try
    {
      protocol1Itf.createExportIdentifier(hints, null);
      fail("createExportIdentifier should throw an exception here");
    }
    catch (InvalidExportIdentifierException e)
    {
      // OK
    }
  }

  public void testCreateExportIdentifier5() throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(TCPIPProtocol.ADDRESS, "localhost");
    hints.put(TCPIPProtocol.PORT, new Integer(1000));
    ExportIdentifier lowerId = new IPExportIdentifier();
    try
    {
      protocol1Itf.createExportIdentifier(hints,
          new ExportIdentifier[]{lowerId});
      fail("createExportIdentifier should throw an exception here");
    }
    catch (InvalidExportIdentifierException e)
    {
      // OK
    }
  }

  public void testCreateExportIdentifier6() throws Exception
  {
    try
    {
      protocol1Itf.createExportIdentifier(null, null);
      fail("createExportIdentifier should throw an exception here");
    }
    catch (InvalidExportIdentifierException e)
    {
      // OK
    }
  }

  public void testCreateExportIdentifier7() throws Exception
  {
    protocol1Itf.createExportIdentifier("localhost", 1000);
  }
}
