/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.queue;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;

public class TestPushPullQueues extends TestCase
{
  private static final int    WARMUP_LOOP               = 2000000;

  private static final String PRIMITIVE_QUEUE_ADL       = "org.objectweb.dream.queue.TestPushPullPrimitive";
  private static final String COMPOSITE_QUEUE_ADL       = "org.objectweb.dream.queue.TestPushPullComposite";
  private static final String BASIC_MESSAGE_MANAGER_ADL = "org.objectweb.dream.message.MessageManager";
  private static final String POOL_MESSAGE_MANAGER_ADL  = "org.objectweb.dream.message.MessageManagerPooling(50)";
  private NumberFormat        groupFormat;
  private NumberFormat        timeFormat;
  private NumberFormat        rateFormat;
  private Component           testComponent;
  private Push                pushItf;
  private Pull                pullItf;

  private MessageManager      messageManagerItf;

  protected void setUp() throws Exception
  {
    groupFormat = new DecimalFormat("###");
    timeFormat = new DecimalFormat("#0.00");
    rateFormat = new DecimalFormat("0.00E0");
    System.out.println();
  }

  public void testFast1() throws Exception
  {
    init(PRIMITIVE_QUEUE_ADL, "org.objectweb.dream.queue.PushPullQueueFast", false);
    runPerf(1, 1000000);
    runPerf(5, 200000);
    runPerf(10, 100000);
  }

  public void testFast2() throws Exception
  {
    init(PRIMITIVE_QUEUE_ADL, "org.objectweb.dream.queue.PushPullQueueFast", true);
    runPerf(1, 1000000);
    runPerf(5, 200000);
    runPerf(10, 100000);
  }

  public void testNotSynchronized1() throws Exception
  {
    init(PRIMITIVE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueueNotSynchronized", false);
    runPerf(1, 1000000);
    runPerf(5, 200000);
    runPerf(10, 100000);
  }

  public void testNotSynchronized2() throws Exception
  {
    init(PRIMITIVE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueueNotSynchronized", true);
    runPerf(1, 1000000);
    runPerf(5, 200000);
    runPerf(10, 100000);
  }

  public void testBlocking_Blocking_FIFO1() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,Blocking,Blocking,FIFO)", false);
    runPerf(1, 1000000);
    runPerf(5, 200000);
    runPerf(10, 100000);
  }

  public void testBlocking_Blocking_FIFO2() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,Blocking,Blocking,FIFO)", true);
    runPerf(1, 1000000);
    runPerf(5, 200000);
    runPerf(10, 100000);
  }

  public void testBlocking_NonBlocking_FIFO1() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,Blocking,NonBlocking,FIFO)", false);
    runPerf(1, 1000000);
    runPerf(5, 200000);
    runPerf(10, 100000);
  }

  public void testBlocking_NonBlocking_FIFO2() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,Blocking,NonBlocking,FIFO)", true);
    runPerf(1, 1000000);
    runPerf(5, 200000);
    runPerf(10, 100000);
  }

  public void testDropping_Blocking_FIFO() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,Dropping,Blocking,FIFO)", true);
    runPerf(1, 1000000);
  }

  public void testDropping_NonBlocking_FIFO() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,Dropping,NonBlocking,FIFO)", true);
    runPerf(1, 1000000);
  }

  public void testExceptionThrowing_Blocking_FIFO() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,ExceptionThrowing,Blocking,FIFO)", true);
    runPerf(1, 1000000);
  }

  public void testExceptionThrowing_NonBlocking_FIFO() throws Exception
  {
    init(
        COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,ExceptionThrowing,NonBlocking,FIFO)", true);
    runPerf(1, 1000000);
  }

  public void testRemoving_Blocking_FIFO() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,Removing,Blocking,FIFO)", true);
    runPerf(1, 1000000);
  }

  public void testRemoving_NonBlocking_FIFO() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,Removing,NonBlocking,FIFO)", true);
    runPerf(1, 1000000);
  }

  public void testBlocking_Blocking_LIFO() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,Blocking,Blocking,LIFO)", true);
    runPerf(1, 1000000);
  }

  public void testBlocking_NonBlocking_LIFO() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,Blocking,NonBlocking,LIFO)", true);
    runPerf(1, 1000000);
  }

  public void testDropping_Blocking_LIFO() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,Dropping,Blocking,LIFO)", true);
    runPerf(1, 1000000);
  }

  public void testDropping_NonBlocking_LIFO() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,Dropping,NonBlocking,LIFO)", true);
    runPerf(1, 1000000);
  }

  public void testExceptionThrowing_Blocking_LIFO() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,ExceptionThrowing,Blocking,LIFO)", true);
    runPerf(1, 1000000);
  }

  public void testExceptionThrowing_NonBlocking_LIFO() throws Exception
  {
    init(
        COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,ExceptionThrowing,NonBlocking,LIFO)", true);
    runPerf(1, 1000000);
  }

  public void testRemoving_Blocking_LIFO() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,Removing,Blocking,LIFO)", true);
    runPerf(1, 1000000);
  }

  public void testRemoving_NonBlocking_LIFO() throws Exception
  {
    init(COMPOSITE_QUEUE_ADL,
        "org.objectweb.dream.queue.PushPullQueue(10,Removing,NonBlocking,LIFO)", true);
    runPerf(1, 1000000);
  }

  private void init(String adl, String queueADL, boolean messagePool) throws Exception
  {
    Map<String, Object> hints = new HashMap<String, Object>();
    if (messagePool) {
      System.out.println("Messages recycled, queue ADL: " + queueADL);
      hints.put("messageManagerADL", POOL_MESSAGE_MANAGER_ADL);
    }
    else {
      System.out.println("Messages created, queue ADL: " + queueADL);
      hints.put("messageManagerADL", BASIC_MESSAGE_MANAGER_ADL);
    }
    
    hints.put("queueADL", queueADL);
    testComponent = (Component) FactoryFactory.getFactory().newComponent(adl,
        hints);
    Fractal.getLifeCycleController(testComponent).startFc();
    pushItf = (Push) testComponent.getFcInterface("push");
    pullItf = (Pull) testComponent.getFcInterface("pull");

    messageManagerItf = (MessageManager) testComponent
        .getFcInterface(MessageManager.ITF_NAME);

    System.err.print(" warming up...");
    warmup();
    System.err.println(" OK");
  }

  private void warmup() throws Exception
  {
    runPerf1(10, WARMUP_LOOP);
  }

  private void runPerf(int burstSize, int nbBurst) throws Exception
  {
    long start = System.currentTimeMillis();
    runPerf1(burstSize, nbBurst);
    printResult(burstSize, nbBurst,
        (System.currentTimeMillis() - start) / 1000D);
  }

  private void runPerf1(int burstSize, int nbBurst) throws PushException,
      PullException
  {
    for (int burst = 0; burst < nbBurst; burst++)
    {
      for (int i = 0; i < burstSize; i++)
      {
        pushItf.push(messageManagerItf.createMessage());
      }
      for (int i = 0; i < burstSize; i++)
      {
        messageManagerItf.deleteMessage(pullItf.pull());
      }
    }
  }

  private void printResult(int burstSize, int nbBurst, double time)
  {
    StringBuffer sb = new StringBuffer();
    sb.append(burstSize * nbBurst);
    sb.append(" messages; group of ");
    addSpace(2, groupFormat.format(burstSize), sb);
    sb.append(" ; finished in ");
    addSpace(5, timeFormat.format(time), sb);
    sb.append("s. rate=").append(
        rateFormat.format((burstSize * nbBurst) / time));
    sb.append(" msg/s");
    System.out.println(sb.toString());
  }

  private void addSpace(int nbChar, String s, StringBuffer toAppendTo)
  {
    for (int i = 0; i < nbChar - s.length(); i++)
    {
      toAppendTo.append(' ');
    }
    toAppendTo.append(s);
  }
}
