/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.dream.control.binding.ContextualBindingController;
import org.objectweb.dream.control.lifecycle.NeedAsyncStartController;
import org.objectweb.dream.control.lifecycle.PrepareStopLifeCycleController;
import org.objectweb.dream.control.logger.Loggable;
import org.objectweb.dream.control.logger.LoggerControllerRegister;
import org.objectweb.dream.util.NullLogger;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.julia.control.binding.ChainedIllegalBindingException;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Base implementation of Dream components. This class is provided for
 * convenience. It is not mandatory for a component implementation to be a
 * subclass of this class. <br>
 * This class provides implementation of <code>BindingController</code>
 * methods. Bindings are stored in a {@link java.util.Map}. When a component
 * has client interfaces, it is recommended to override (at least) the
 * {@link ContextualBindingController#bindFc(String, Object) bindFc }method.
 */
public abstract class AbstractComponent
    implements
      LifeCycleController,
      NeedAsyncStartController,
      PrepareStopLifeCycleController,
      ContextualBindingController,
      Loggable
{

  /** the {@link Component }controller interface reference of this component */
  public Component  weaveableC;

  /** the functional logger of this component */
  protected Logger  logger           = NullLogger.NULL_LOGGER;

  /**
   * the controller logger associated with the life cycle controller of this
   * component
   */
  protected Logger  lifeCycleLogger  = NullLogger.NULL_LOGGER;

  /**
   * the controller logger associated with the binding controller of this
   * component
   */
  protected Logger  bindingLogger    = NullLogger.NULL_LOGGER;

  /** this hashtable contain all bindings of the component. */
  private Map<String, Object>       bindings;

  protected boolean fcNeedAsyncStart = false;

  /**
   * this attribute give the state of Dream Component. <br>
   * the fcSstate value is {@link LifeCycleController#STARTED}
   * {@link LifeCycleController#STOPPED}
   */
  protected String  fcState;

  /** A boolean that indicates whether this is the first start or not. */
  private boolean   firstStart       = true;

  /**
   * Constructor method create the hashtable of bindings and initialize fcState
   * to LifeCycleController.STOPPED.
   */
  public AbstractComponent()
  {
    bindings = new HashMap<String, Object>();
    fcState = LifeCycleController.STOPPED;
  }

  // ---------------------------------------------------------------------------
  // Methods extending the component lifecycle
  // ---------------------------------------------------------------------------

  /**
   * Initializes this component. This method is called during initialization of
   * the component controllers. <br>
   * This method can be overridden by sub classes as follows:
   * 
   * <pre>
   *  
   *   
   *   protected void initComponent
   *   {
   *     super.initComponent;
   *     // initialization code here ...
   *   }
   *   
   *   
   *   
   * </pre>
   * 
   * @throws InitializationException if an error occurs.
   */
  protected void initComponent() throws InitializationException
  {
    try
    {
      LoggerControllerRegister lcr = (LoggerControllerRegister) weaveableC
          .getFcInterface("/logger-controller-register");
      lcr.register("impl", this);
      lcr.register("life-cycle", this);
      lcr.register("binding", this);
      logger.log(BasicLevel.DEBUG, "Component initialized.");
    }
    catch (NoSuchInterfaceException e)
    {
      // ignore
    }
  }

  /**
   * This method is called the first time the component is started.
   * 
   * @param componentItf the {@link Component }interface of this component.
   */
  protected void beforeFirstStart(Component componentItf)
      throws IllegalLifeCycleException
  {

  }

  // ---------------------------------------------------------------------------
  // Implementation of the Loggable interface
  // ---------------------------------------------------------------------------

  /**
   * @see Loggable#setLogger(String, Logger)
   */
  public void setLogger(String name, Logger l)
  {
    if (name.equals("impl"))
    {
      logger = l;
    }
    else if (name.equals("life-cycle"))
    {
      lifeCycleLogger = l;
    }
    else if (name.equals("binding"))
    {
      bindingLogger = l;
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the NeedAsyncStartController interface
  // ---------------------------------------------------------------------------

  /**
   * @see NeedAsyncStartController#getFcNeedAsyncStart()
   */
  public boolean getFcNeedAsyncStart()
  {
    return fcNeedAsyncStart;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the PrepareStopLifeCycleController interface
  // ---------------------------------------------------------------------------

  /**
   * @see PrepareStopLifeCycleController#prepareStopFc()
   */
  public void prepareStopFc() throws IllegalLifeCycleException
  {
    logger.log(BasicLevel.DEBUG, "Going to stop.");
  }

  // ---------------------------------------------------------------------------
  // Implementation of the LifeCycleController interface
  // ---------------------------------------------------------------------------

  /**
   * @see LifeCycleController#getFcState()
   */
  public String getFcState()
  {
    if (lifeCycleLogger.isLoggable(BasicLevel.DEBUG))
    {
      lifeCycleLogger.log(BasicLevel.DEBUG, "getFcState: " + fcState);
    }
    return fcState;
  }

  /**
   * @see LifeCycleController#stopFc()
   */
  public void stopFc() throws IllegalLifeCycleException
  {
    fcState = LifeCycleController.STOPPED;
    lifeCycleLogger.log(BasicLevel.DEBUG, "stopped");
  }

  /**
   * @see LifeCycleController#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    if (firstStart)
    {
      beforeFirstStart(weaveableC);
      firstStart = false;
    }
    fcState = LifeCycleController.STARTED;
    lifeCycleLogger.log(BasicLevel.DEBUG, "started");
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see ContextualBindingController#lookupFc(String)
   */
  public Object lookupFc(String clientItfName) throws NoSuchInterfaceException
  {
    synchronized (bindings)
    {
      if (bindingLogger.isLoggable(BasicLevel.DEBUG))
      {
        bindingLogger.log(BasicLevel.DEBUG, "lookupFc on interface \""
            + clientItfName + "\"");
      }
      return bindings.get(clientItfName);
    }
  }

  /**
   * @see ContextualBindingController#bindFc(String, Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    synchronized (bindings)
    {
      if (bindingLogger.isLoggable(BasicLevel.DEBUG))
      {
        bindingLogger.log(BasicLevel.DEBUG, "bindFc on interface \""
            + clientItfName + "\"");
      }
      if (clientItfName.equals("component"))
      {
        weaveableC = (Component) serverItf;
        try
        {
          initComponent();
        }
        catch (InitializationException e)
        {
          logger.log(BasicLevel.ERROR,
              "An error occurred while initializing the component", e);
          throw new ChainedIllegalBindingException(e, weaveableC, null,
              "component", null,
              "An error occurred while initializing component");
        }
      }
      else
      {
        bindings.put(clientItfName, serverItf);
      }
    }
  }

  /**
   * @see ContextualBindingController#bindFc(String, Object, Object)
   */
  public void bindFc(String clientItfName, Object serverItf, Object hints)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    synchronized (bindings)
    {
      if (bindingLogger.isLoggable(BasicLevel.DEBUG))
      {
        bindingLogger.log(BasicLevel.DEBUG, "bindFc on interface \""
            + clientItfName + "\"");
      }
      if (clientItfName.equals("component"))
      {
        weaveableC = (Component) serverItf;
        try
        {
          initComponent();
        }
        catch (InitializationException e)
        {
          logger.log(BasicLevel.ERROR,
              "An error occurred while initializing the component", e);
          throw new ChainedIllegalBindingException(e, weaveableC, null,
              "component", null,
              "An error occurred while initializing component");
        }
      }
      else
      {
        bindings.put(clientItfName, serverItf);
      }
    }
  }

  /**
   * @see ContextualBindingController#unbindFc(String)
   */
  public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
      IllegalBindingException, IllegalLifeCycleException
  {
    synchronized (bindings)
    {
      if (bindingLogger.isLoggable(BasicLevel.DEBUG))
      {
        bindingLogger.log(BasicLevel.DEBUG, "unbindIf on interface \""
            + clientItfName + "\"");
      }
      bindings.remove(clientItfName);
    }
  }
}