/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.activity.manager;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.control.activity.task.IllegalTaskException;
import org.objectweb.dream.control.activity.task.Task;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Routes {@link SchedulerManager }calls to the correct scheduler interface
 * depending on hints.
 */
public class MetaSchedulerManagerImpl extends AbstractComponent
    implements
      SchedulerManager
{

  static final String         THREAD_HINTS_KEY            = "thread";
  static final String         MONO_THREAD_HINTS_VALUE     = "mono";
  static final String         POOL_THREAD_HINTS_VALUE     = "pool";
  static final String         PERIODIC_HINTS_KEY          = "period";
  Map<Task, SchedulerManager> taskScheduler               = new HashMap<Task, SchedulerManager>();

  // ---------------------------------------------------------------------------
  // client interfaces
  // ---------------------------------------------------------------------------

  static final String         TPT_SCHEDULER_ITF_NAME      = "tpt-scheduler-manager";
  SchedulerManager            tptSchedulerItf;
  static final String         TPPT_SCHEDULER_ITF_NAME     = "tppt-scheduler-manager";
  SchedulerManager            tpptSchedulerItf;
  static final String         PERIODIC_SCHEDULER_ITF_NAME = "periodic-scheduler-manager";
  SchedulerManager            periodicSchedulerItf;

  // ---------------------------------------------------------------------------
  // Implementation of the SchedulerManager interface
  // ---------------------------------------------------------------------------

  /**
   * @see SchedulerManager#scheduleTask(Task, Map)
   */
  public Object scheduleTask(Task task, Map hints) throws IllegalTaskException
  {
    SchedulerManager itf;
    Object h = hints.get(PERIODIC_HINTS_KEY);
    if (h != null)
    {
      itf = periodicSchedulerItf;
    }
    else
    {
      String s = (String) hints.get(THREAD_HINTS_KEY);
      if (s == null || MONO_THREAD_HINTS_VALUE.equals(s))
      {
        itf = tptSchedulerItf;
      }
      else if (POOL_THREAD_HINTS_VALUE.equals(s))
      {
        itf = tpptSchedulerItf;
      }
      else
      {
        throw new IllegalTaskException(task, "Invalid hints");
      }
    }
    taskScheduler.put(task, itf);
    return itf.scheduleTask(task, hints);
  }

  /**
   * @see SchedulerManager#unscheduleTask(Task)
   */
  public void unscheduleTask(Task task) throws IllegalTaskException
  {
    SchedulerManager itf = taskScheduler.remove(task);
    if (itf == null)
    {
      throw new IllegalTaskException(task, "Unknown task");
    }
    itf.unscheduleTask(task);
  }

  /**
   * @see org.objectweb.dream.control.activity.scheduler.InterruptTask#interruptTask(Task)
   */
  public void interruptTask(Task task) throws IllegalTaskException
  {
    SchedulerManager itf = taskScheduler.get(task);
    if (itf == null)
    {
      throw new IllegalTaskException(task, "Unknown task");
    }
    itf.interruptTask(task);
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{TPT_SCHEDULER_ITF_NAME, TPPT_SCHEDULER_ITF_NAME,
        PERIODIC_SCHEDULER_ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(TPT_SCHEDULER_ITF_NAME))
    {
      tptSchedulerItf = (SchedulerManager) serverItf;
    }
    else if (clientItfName.equals(TPPT_SCHEDULER_ITF_NAME))
    {
      tpptSchedulerItf = (SchedulerManager) serverItf;
    }
    else if (clientItfName.equals(PERIODIC_SCHEDULER_ITF_NAME))
    {
      periodicSchedulerItf = (SchedulerManager) serverItf;
    }
  }
}
