/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.activity.manager;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

import org.objectweb.dream.control.activity.scheduler.InterruptTask;
import org.objectweb.dream.control.activity.scheduler.MultiThreadForwarderSchedulerImpl;
import org.objectweb.dream.control.activity.scheduler.Scheduler;
import org.objectweb.dream.control.activity.task.IllegalTaskException;
import org.objectweb.dream.control.activity.task.Task;
import org.objectweb.dream.control.activity.task.thread.ThreadPoolOverflowException;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.LifeCycleController;

/**
 * Scheduler manager that schedules each task with a
 * {@link MultiThreadForwarderSchedulerImpl} and initially no thread.
 */
public class ThreadPoolPerTaskSchedulerManagerImpl
    extends
      AbstractSchedulerManager implements ThreadPoolManager, EndOfThread
{
  static final String            THREAD_PER_TASK_END_OF_THREAD_ITF_NAME = "tppt-end-of-thread";
  Map<Task, TaskState>           scheduledTask                          = new HashMap<Task, TaskState>();
  Map<Task, TaskState>           schedulerThread                        = new HashMap<Task, TaskState>();
  LinkedList<WaitingThreadState> waitingThreadStates                    = new LinkedList<WaitingThreadState>();

  // ---------------------------------------------------------------------------
  // Attributes fields
  // ---------------------------------------------------------------------------

  long                           reviveTimeout                          = 2000;

  // ---------------------------------------------------------------------------
  // Implementation of the SchedulerManager interface
  // ---------------------------------------------------------------------------

  /**
   * @see SchedulerManager#scheduleTask(Task, Map)
   */
  public synchronized Object scheduleTask(Task task, Map hints)
      throws IllegalTaskException
  {
    if (scheduledTask.containsKey(task))
    {
      throw new IllegalTaskException(task, "Task already scheduled");
    }

    // add and bind scheduler
    Scheduler scheduler = new MultiThreadForwarderSchedulerImpl();
    ThreadPoolManager ctrItf = null;
    try
    {
      contentControllerItf.addFcSubComponent(scheduler);
      ((BindingController) scheduler).bindFc(Task.ITF_NAME, task);
      ((BindingController) scheduler).bindFc(EndOfTask.ITF_NAME,
          contentControllerItf.getFcInternalInterface(EndOfTask.ITF_NAME));
      ((BindingController) scheduler).bindFc("logger", logger);
      ctrItf = (ThreadPoolManager) contentControllerItf
          .getFcInternalInterface(ThreadPoolManager.ITF_NAME);
    }
    catch (Exception e)
    {
      Error.bug(logger, e);
    }

    TaskState taskState = new TaskState(task, scheduler);
    scheduledTask.put(task, taskState);
    return ctrItf;
  }

  /**
   * @see SchedulerManager#unscheduleTask(Task)
   */
  public synchronized void unscheduleTask(Task task)
      throws IllegalTaskException
  {
    TaskState state = getTask(task);
    state.running = false;
    try
    {
      // EndofTask notified to task manager, just unbind scheduler
      ((BindingController) state.scheduler).unbindFc(Task.ITF_NAME);
      // wait for the EndOfThread to remove components.
    }
    catch (Exception e)
    {
      Error.bug(logger, e);
    }
  }

  /**
   * @see InterruptTask#interruptTask(Task)
   */
  public synchronized void interruptTask(Task task) throws IllegalTaskException
  {
    TaskState state = getTask(task);
    ((InterruptTask) state.scheduler).interruptTask(task);
  }

  // ---------------------------------------------------------------------------
  // Implementation of the EndOfThread interface
  // ---------------------------------------------------------------------------

  /**
   * @see EndOfThread#threadEnded(Task)
   */
  public boolean threadEnded(Task thread)
  {
    TaskState state;
    synchronized (this)
    {
      state = schedulerThread.get(thread);
    }
    if (state == null)
    {
      Error.bug(logger);
    }
    synchronized (state)
    {
      if (state == null)
      {
        Error.bug(logger);
      }
      state.threads.remove(thread);
      schedulerThread.remove(thread);
      if (state.running == false && state.threads.isEmpty())
      {
        try
        {
          ((BindingController) thread).unbindFc(Scheduler.ITF_NAME);
          contentControllerItf.removeFcSubComponent(state.scheduler);
        }
        catch (Exception e)
        {
          Error.bug(logger, e);
        }

      }
    }
    WaitingThreadState threadState = new WaitingThreadState(thread);
    synchronized (waitingThreadStates)
    {
      waitingThreadStates.addFirst(threadState);
    }
    try
    {
      synchronized (threadState)
      {
        if (!threadState.revive)
        {
          threadState.wait(reviveTimeout);
        }
      }
    }
    catch (InterruptedException e)
    {
      // do nothing
    }
    finally
    {
      synchronized (waitingThreadStates)
      {
        if (!threadState.revive)
        {
          waitingThreadStates.remove(threadState);
        }
        else
        {
          return true;
        }
      }
    }
    try
    {
      threadManagerItf.removeThread(thread);
    }
    catch (Exception e)
    {
      Error.bug(logger, e);
    }
    return false;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the ThreadPoolManager interface
  // ---------------------------------------------------------------------------

  /**
   * @see ThreadPoolManager#getNbActiveThreads(Task)
   */
  public int getNbActiveThreads(Task task) throws IllegalTaskException
  {
    TaskState state = getTask(task);
    return state.threads.size();
  }

  /**
   * @see ThreadPoolManager#addThread(Task)
   */
  public synchronized void addThread(Task task)
      throws ThreadPoolOverflowException, IllegalTaskException
  {
    TaskState state = getTask(task);
    Task thread = null;
    synchronized (waitingThreadStates)
    {
      if (!waitingThreadStates.isEmpty())
      {
        WaitingThreadState threadState = waitingThreadStates.removeLast();
        thread = threadState.thread;
        try
        {
          ((BindingController) thread).bindFc(Scheduler.ITF_NAME,
              state.scheduler);
        }
        catch (Exception e)
        {
          Error.bug(logger, e);
        }
        state.threads.add(thread);
        schedulerThread.put(thread, state);
        synchronized (threadState)
        {
          threadState.revive = true;
          threadState.notify();
        }
        return;
      }
    }
    thread = threadManagerItf.addThread();
    try
    {
      ((BindingController) thread).bindFc(Scheduler.ITF_NAME, state.scheduler);
      ((BindingController) thread).bindFc(EndOfThread.ITF_NAME,
          contentControllerItf
              .getFcInternalInterface(THREAD_PER_TASK_END_OF_THREAD_ITF_NAME));
      state.threads.add(thread);
      schedulerThread.put(thread, state);
      ((LifeCycleController) thread).startFc();
    }
    catch (Exception e)
    {
      Error.bug(logger, e);
    }
  }

  // ---------------------------------------------------------------------------
  // Utility method
  // ---------------------------------------------------------------------------

  private TaskState getTask(Task task) throws IllegalTaskException
  {
    TaskState state = scheduledTask.get(task);
    if (state == null)
    {
      throw new IllegalTaskException(task, "Unknown task");
    }
    return state;
  }

  // ---------------------------------------------------------------------------
  // inner classes
  // ---------------------------------------------------------------------------

  class TaskState
  {
    boolean   running = true;
    Task      task;
    Scheduler scheduler;
    Set<Task> threads = new HashSet<Task>();

    TaskState(Task task, Scheduler scheduler)
    {
      this.task = task;
      this.scheduler = scheduler;
    }
  }

  class WaitingThreadState
  {
    boolean revive = false;
    Task    thread;

    WaitingThreadState(Task thread)
    {
      this.thread = thread;
    }
  }
}
