/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec;

import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.adl.Reconfiguration;
import org.objectweb.dream.adl.ReconfigurationFactory;
import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.util.Fractal;

/**
 * Basic implementation of codec manager. Use a {@link CodecRepository }to
 * associate codec name with ADL.
 */
public class CodecManagerImpl extends AbstractComponent implements CodecManager
{

  /**
   * The name of the client interface bound to the component interface of the
   * composite containing codecs.
   */
  public static final String  CODECS_COMPONENT_ITF_NAME = "codecs-component";

  protected CodecRepository   codecRepositoryItf;
  protected Component         codecsCompositeItf;
  protected ContentController codecsCCItf;

  /**
   * @see CodecManager#addCodec(String, Map)
   */
  public void addCodec(String codecName, Map<String, Object> context)
      throws CodecManagerException
  {
    Component codec = findCodec(codecName);
    if (codec != null)
    {
      return;
    }
    String codecADL = codecRepositoryItf.getCodecADL(codecName);
    if (codecADL == null)
    {
      throw new CodecManagerException(codecName, "Unknown codec.");
    }
    createCodec(codecName, codecADL, context);
  }

  /**
   * @see CodecManager#removeCodec(String)
   */
  public void removeCodec(String codecName) throws CodecManagerException
  {
    // NOT IMPLEMENTED Auto-generated method stub

  }

  /**
   * @see CodecManager#getCodec(String)
   */
  public Component getCodec(String codecName) throws CodecManagerException
  {
    Component codec = findCodec(codecName);
    if (codec == null)
    {
      throw new CodecManagerException(codecName, "Unknown codec");
    }
    return codec;
  }

  // ---------------------------------------------------------------------------
  // Utility method
  // ---------------------------------------------------------------------------

  /**
   * Finds and returns a codec component with the specified name. If the codec
   * can't be found, returns <code>null</code>.
   */
  protected Component findCodec(String codecName)
  {
    Component[] codecs = codecsCCItf.getFcSubComponents();
    for (int i = 0; i < codecs.length; i++)
    {
      Component component = codecs[i];
      String name;
      try
      {
        name = Fractal.getNameController(component).getFcName();
      }
      catch (NoSuchInterfaceException e)
      {
        break;
      }
      if (codecName.equals(name))
      {
        return component;
      }
    }
    return null;
  }

  /**
   * Creates a codec and adds it in the codecs composite
   */
  protected void createCodec(String codecName, String codecADL,
      Map<String, Object> context) throws CodecManagerException
  {
    context.put("codecName", codecName);
    try
    {
      Reconfiguration reconf = ReconfigurationFactory.getReconfiguration();
      reconf.addComponents(codecsCompositeItf, codecADL, context);
    }
    catch (ADLException e)
    {
      throw new CodecManagerException(codecName,
          "Unable to create codec caused by :" + e.toString());
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{CodecRepository.ITF_NAME, CODECS_COMPONENT_ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    if (clientItfName.equals(CodecRepository.ITF_NAME))
    {
      codecRepositoryItf = (CodecRepository) serverItf;
    }
    else if (clientItfName.equals(CODECS_COMPONENT_ITF_NAME))
    {
      codecsCompositeItf = (Component) serverItf;
      try
      {
        codecsCCItf = Fractal.getContentController(codecsCompositeItf);
      }
      catch (NoSuchInterfaceException e)
      {
        throw new IllegalBindingException("The " + CODECS_COMPONENT_ITF_NAME
            + " client interface must be bound to a composite component");
      }
    }
    super.bindFc(clientItfName, serverItf);
  }
}
